/**
 * Problem: bombs
 * Correct solution (must be OK).
 * Takes O(M N) time.
 * Wave algorithm:
 *  0) Expand {s} over empty cells.
 *  1) Dilate new cells by doing P 4-side walks and R 8-side walks (in "waves" over unvisited cells).
 *  2) Expand over empty cells the new cells from step 1.
 *  3) Repeat from step 1, using cells added both on step 1 and 2.
 * Author: stgatilov
 */
//#pragma comment(linker, "/STACK:20000000")
#include <vector>
#include <list>
#include <map>
#include <set>
#include <deque>
#include <stack>
#include <bitset>
#include <algorithm>
#include <functional>
#include <numeric>
#include <utility>
#include <sstream>
#include <iostream>
#include <iomanip>
#include <cstdio>
#include <cmath>
#include <cstring>
#include <cassert>
#include <cstdlib>
#include <ctime>
#include <cstdint>
#include <unordered_set>
#include <cinttypes>
#include <array>
#include <memory>

using namespace std;
typedef long long int64;
#ifdef HOME
    #define E(c) cerr<<#c
    #define Eo(x) cerr<<#x<<" = "<<(x)<<endl
    #define Ef(...) fprintf(stderr, __VA_ARGS__)
#else
    #define E(c) ((void)0)
    #define Eo(x) ((void)0)
    #define Ef(...) ((void)0)
#endif

const int MAXN = 1536;

struct Pos {
    int r, c;
};
const Pos dirs[8] = {
    Pos{1, 0}, Pos{-1, 0}, Pos{0, 1}, Pos{0, -1},
    Pos{-1, -1}, Pos{-1, 1}, Pos{1, -1}, Pos{1, 1}
};
Pos operator+ (const Pos &a, const Pos &b) {
    return Pos{a.r + b.r, a.c + b.c};
}
bool operator== (const Pos &a, const Pos &b) {
    return a.r == b.r && a.c == b.c;
}

template<class T> struct Field {
    T a[MAXN][MAXN+1];
    T& operator[](const Pos &p) { return a[p.r][p.c]; }
    T operator[](const Pos &p) const { return a[p.r][p.c]; }
};
typedef Field<int> IField;
typedef Field<char> BField;
typedef Field<Pos> PField;

int m, n, bp, br;
BField field;
Pos s, t;

void Print(const IField &f, const vector<Pos> *wave = 0) {
    for (int i = 0; i < m; i++) {
        for (int j = 0; j < n; j++) {
            if (wave && count(wave->begin(), wave->end(), Pos{i, j})) Ef(" w");
            else if (f.a[i][j] > 1000000000) Ef(" X");
            else Ef(" %d", f.a[i][j]);
        }
        Ef("\n");
    }
    Ef("\n");
}

bool IsOut(const Pos &a) {
    return a.r < 0 || a.c < 0 || a.r >= m || a.c >= n;
}
Pos StepTo(const Pos &a, const Pos &b) {
    if (a.r != b.r) return a + Pos{b.r < a.r ? -1 : 1, 0};
    if (a.c != b.c) return a + Pos{0, b.c < a.c ? -1 : 1};
    return a;
}

IField dist;
PField father;

vector<Pos> AddEmptyCells(vector<Pos> &&wstart) {
    vector<Pos> wcurr = move(wstart);
    vector<Pos> wnext;
    vector<Pos> wnew = wcurr;
    int cnt;
    do {
        cnt = wcurr.size();
        wnext.clear();
        for (Pos p : wcurr) {
            for (int d = 0; d < 4; d++) {
                Pos np = p + dirs[d];
                if (IsOut(np)) continue;
                if (field[np] != '.') continue;
                if (dist[np] < 1000000000) continue;
                dist[np] = dist[p];
                father[np] = p;
                wnext.push_back(np);
                wnew.push_back(np);
            }
        }
        swap(wcurr, wnext);
    } while (cnt);
    return wnew;
}

vector<Pos> DilateBombs(vector<Pos> &&wstart) {
    vector<Pos> wcurr = move(wstart);
    vector<Pos> wnext;
    vector<Pos> wnew;
    for (int i = 0; i < br + bp/2; i++) {
        wnext.clear();
        for (Pos p : wcurr) {
            for (int d = 0; d < (i < br ? 4 : 8); d++) {
                Pos np = p + dirs[d];
                if (IsOut(np)) continue;
                if (dist[np] < 1000000000) continue;
                dist[np] = dist[p];
                father[np] = p;
                wnext.push_back(np);
                wnew.push_back(np);
            }
        }
        swap(wcurr, wnext);
    }
    for (Pos p : wnew)
        dist[p]++;
    return wnew;
}

int main() {
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);

    scanf("%d%d%d%d", &m, &n, &bp, &br);
    assert(bp % 2 == 0);
    for (int i = 0; i < m; i++)
        scanf("%s", field.a[i]);
    for (int i = 0; i < m; i++)
        for (int j = 0; j < n; j++) {
            if (field.a[i][j] == 'S') {
                s = Pos{i, j};
                field[s] = '.';
            }
            if (field.a[i][j] == 'E') {
                t = Pos{i, j};
                field[t] = '.';
            }
        }
//    Ef("%d %d  %d %d\n", s.r, s.c, t.r, t.c);

    memset(&dist, 63, sizeof(dist));
    dist[s] = 0;
    vector<Pos> wave(1, s);
    wave = AddEmptyCells(move(wave));
    //Print(dist, &wave);
    for (int bombs = 0; dist[t] > 1000000000; bombs++) {
        //Ef("Bomb %d:\n", bombs);
        wave = DilateBombs(move(wave));
        //Print(dist, &wave);
        wave = AddEmptyCells(move(wave));
        //Print(dist, &wave);
    }

    int ans = dist[t];
    assert(ans < 1000000000);
    Pos curr = t;
    vector<Pos> bombs, dest;
    while (!(curr == s)) {
        Pos prev = father[curr];
        if (dist[prev] < dist[curr]) {
            bombs.push_back(prev);
            dest.push_back(curr);
        }
        curr = prev;
    }
    reverse(bombs.begin(), bombs.end());
    reverse(dest.begin(), dest.end());

    vector<Pos> strel;
    int rad = br + bp/2;
    for (int i = -rad; i <= rad; i++)
        for (int j = -rad; j <= rad; j++)
            if (abs(i) + abs(j) <= bp + br)
                strel.push_back(Pos{i, j});
    for (int i = 0; i < bombs.size(); i++) {
        Pos p = bombs[i], q = dest[i];
        while (1) {
            assert(!(p==q));
            if (field[StepTo(p, q)] == '@')
                break;
            p = StepTo(p, q);
        }
        bombs[i] = p;
        for (Pos delta : strel)
            if (!IsOut(p + delta))
                field[p + delta] = 'b';
    }

    printf("%d\n", ans);
    for (auto b : bombs)
        printf("%d %d\n", b.r + 1, b.c + 1);

    return 0;
}
