/**
 * Problem: fixheap
 * Correct solution (must be OK)
 * Author: stgatilov
 *
 * Solution: DP in O(N)
 * Optimizes min using sliding window (increasing queue).
 * Special cases (when some number matches) are handled explicitly --- at most 2N edges.
 * Stores "backedges" only for the next 256 elements, drops old backedges.
 *
 * O(N) memory usage consists of:
 *    N bytes for restoring DP solution
 *    N bytes for storing input data
 *    (probably also up to N bytes for low-level file buffer --- can't control it)
 */

//#pragma comment(linker, "/STACK:20000000")
#include <vector>
#include <list>
#include <map>
#include <set>
#include <deque>
#include <stack>
#include <bitset>
#include <algorithm>
#include <functional>
#include <numeric>
#include <utility>
#include <sstream>
#include <iostream>
#include <iomanip>
#include <cstdio>
#include <cmath>
#include <cstring>
#include <cassert>
#include <cstdlib>
#include <ctime>
#include <cstdint>
#include <unordered_set>
#include <cinttypes>
#include <array>
#include <memory>

using namespace std;
typedef long long int64;
#ifdef HOME
    #define E(c) cerr<<#c
    #define Eo(x) cerr<<#x<<" = "<<(x)<<endl
    #define Ef(...) fprintf(stderr, __VA_ARGS__)
#else
    #define E(c) ((void)0)
    #define Eo(x) ((void)0)
    #define Ef(...) ((void)0)
#endif

//important constant! do not change!
const int MAXBLOCK = UINT8_MAX;
//define limits: can safely increase
const int MAXN = (1<<24) + 10;
const int MAXANS = 2 * (MAXN / MAXBLOCK + 10);
const int WINDOW = 512;

int n;
uint8_t idata[MAXN];
int ans;
uint32_t answer[MAXANS];

void ReadInput() {
    FILE *f = fopen("input.bin", "rb");
    fseek(f, 0, SEEK_END);
    n = ftell(f);
    fseek(f, 0, SEEK_SET);
    int res = fread(idata, 1, n, f);
    assert(res == n);
    fclose(f);
}

void WriteOutput() {
    FILE *f = fopen("output.bin", "wb");
    int res = fwrite(answer, 4, ans, f);
    assert(res == ans);
    fclose(f);
}

void CheckAndSet(int address, int value) {
    if (idata[address] == value)
        return;
    uint32_t x = value;
    x <<= 24;
    x |= address;
    answer[ans++] = x;
}

//DP arrays
int res[WINDOW];                //only last 512 elements
uint8_t father[MAXN+1];         //store all!
//for getting minimum on segment
struct ResAndIdx {
    int res;
    int idx;
};
deque<ResAndIdx> minQueue;      //up to 256 last elements
//special cases: back edges
vector<int> backEdges[WINDOW];  //up to 512 last elements

void Solve() {
    //backedge from i = 0 (won't add it in loop)
    backEdges[int(idata[0]) + 2].push_back(0);

    res[0] = 0;
    father[0] = -1;
    for (int i = 1; i <= n; i++) {
        ResAndIdx best{1000000000, -1};
        //min over window
        if (!minQueue.empty()) {
            best = minQueue.front();
            best.res += 2;
        }

        //advance window
        if (!minQueue.empty() && minQueue.front().idx == i - (MAXBLOCK+2))
            minQueue.pop_front();
        int addedRes = res[(i-1) % WINDOW];
        while (!minQueue.empty() && minQueue.back().res >= addedRes)
            minQueue.pop_back();
        minQueue.push_back(ResAndIdx{addedRes, i-1});

        //process special edges
        auto &be = backEdges[i % WINDOW];
        for (int j = -1; j < (int)be.size(); j++) {
            int from;
            if (j < 0) {
                from = i - idata[i-1] - 2;
                if (from < 0) continue;
            }
            else {
                from = be[j];
            }
            ResAndIdx tmp{res[from % WINDOW] + (idata[from] != i-from-2) + (idata[i-1] != i-from-2), from};
            if (best.res > tmp.res)
                best = tmp;
        }
        be.clear();

        res[i % WINDOW] = best.res;
        father[i] = i - best.idx - 2;

        //add back edge from current position
        if (i < n) {
            int len = idata[i];
            int npos = i + len + 2;
            if (npos <= n)
                backEdges[npos % WINDOW].push_back(i);
        }
    }

    int finalRes = res[n % WINDOW];
    assert(finalRes < 1000000000);
    assert(finalRes <= MAXANS);
    ans = 0;
    int curr = n;
    while (curr != 0) {
        int b = father[curr]+2;
        CheckAndSet(curr-1, b-2);
        CheckAndSet(curr-b, b-2);
        curr -= b;
    }
    assert(ans == finalRes);
}

int main(int argc, char **argv) {
    ReadInput();
    Solve();
    WriteOutput();
    return 0;
}
