/**
 * Problem: treeflow2
 * Correct solution (must be OK)
 * Author: stgatilov
 *
 * Augmenting path solution.
 * Path capacity and push is done with heavy-light decomposition.
 * Time: O(N log^2 N)
 */

#pragma comment(linker, "/STACK:20000000")
#include <vector>
#include <list>
#include <map>
#include <set>
#include <deque>
#include <stack>
#include <bitset>
#include <algorithm>
#include <functional>
#include <numeric>
#include <utility>
#include <sstream>
#include <iostream>
#include <iomanip>
#include <cstdio>
#include <cmath>
#include <cstring>
#include <cassert>
#include <cstdlib>
#include <ctime>
#include <cstdint>
#include <unordered_set>
#include <cinttypes>
#include <array>
#include <memory>

using namespace std;
typedef long long int64;
#ifdef HOME
    #define E(c) cerr<<#c
    #define Eo(x) cerr<<#x<<" = "<<(x)<<endl
    #define Ef(...) fprintf(stderr, __VA_ARGS__)
#else
    #define E(c) ((void)0)
    #define Eo(x) ((void)0)
    #define Ef(...) ((void)0)
#endif

const int MAXN = 200<<10;
const int INFCAP = 1000000000;

struct Edge {
    int v;
    int cap, cost;
};

int n;
vector<Edge> nbr[MAXN];
int par[MAXN], pcap[MAXN];

struct Leaf {
    int v;
    int64 cost;
};
bool operator< (const Leaf &a, const Leaf &b) {
    return a.cost < b.cost;
}
int k;
Leaf leaves[MAXN];
int weight[MAXN];
int order[MAXN], ordk;

void DFS(int u, int f, int fcap, int64 sumCost) {
    par[u] = f;
    pcap[u] = fcap;
    weight[u] = 1;
    int sons = 0;
    for (int i = 0; i < nbr[u].size(); i++) {
        Edge e = nbr[u][i];
        if (e.v == f)
            continue;
        DFS(e.v, u, e.cap, sumCost + e.cost);
        sons++;
        weight[u] += weight[e.v];
    }
    if (sons == 0) {
        Leaf l = {u, sumCost};
        leaves[k++] = l;
    }
    order[ordk++] = u;
}

struct Line {
    vector<int> verts;
    int sz;
    vector<int> trmin, tradd;

    void Update(int v);
    void Actualize(int v);
    int Rec(int v, int ql, int qr, int qadd, int l, int r);
};
bool pheavy[MAXN];
int hidx[MAXN], hpos[MAXN];
vector<Line> lines;

void Line::Actualize(int v) {
    trmin[2*v+1] += tradd[v];
    trmin[2*v+2] += tradd[v];
    tradd[2*v+1] += tradd[v];
    tradd[2*v+2] += tradd[v];
    tradd[v] = 0;
}
void Line::Update(int v) {
    trmin[v] = min(trmin[2*v+1], trmin[2*v+2]);
}
int Line::Rec(int v, int ql, int qr, int qadd, int l, int r) {
    if (ql >= r || qr <= l)
        return INFCAP;
    if (ql <= l && qr >= r) {
        trmin[v] += qadd;
        tradd[v] += qadd;
        return trmin[v];
    }
    Actualize(v);
    int m = (l + r) >> 1;
    int resl = Rec(2*v+1, ql, qr, qadd, l, m);
    int resr = Rec(2*v+2, ql, qr, qadd, m, r);
    Update(v);
    return min(resl, resr);
}

int Query(int v, int add) {
    int res = INFCAP;
    while (par[v] >= 0) {
        int li = hidx[v];
        int lp = hpos[v];
        if (li < 0 || lp == lines[li].verts.size()-1) {
            assert(!pheavy[v]);
            pcap[v] += add;
            res = min(res, pcap[v]);
            v = par[v];
        }
        else {
            Line &l = lines[li];
            int lend = l.verts.size() - 1;
            int lmin = l.Rec(0, lp, lend, add, 0, l.sz);
            res = min(res, lmin);
            v = l.verts[lend];
        }
    }
    return res;
}

int main(int argc, char **argv) {
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);

    scanf("%d", &n);
    for (int i = 0; i < n-1; i++) {
        int a, b, c, w;
        scanf("%d%d%d%d", &a, &b, &c, &w);
        a--;  b--;
        Edge e1 = {b, c, w};
        nbr[a].push_back(e1);
        Edge e2 = {a, c, w};
        nbr[b].push_back(e2);
    }

    DFS(0, -1, -1, 0);
    Ef("%d/%d leaves\n", k, n);
    sort(leaves, leaves + k);

    for (int i = 0; i < n; i++) {
        for (int j = 0; j < nbr[i].size(); j++) {
            int v = nbr[i][j].v;
            if (v == par[i])
                continue;
            if (weight[v]*2 > weight[i])
                pheavy[v] = true;
        }
    }
    memset(hidx, -1, sizeof(hidx));
    for (int i = 0; i < n; i++) {
        int v = order[i];
        if (hidx[v] >= 0 || !pheavy[v])
            continue;
        Line l;
        for (int j = 0; ; j++) {
            l.verts.push_back(v);
            hidx[v] = lines.size();
            hpos[v] = j;
            if (!pheavy[v])
                break;
            v = par[v];
        }
        lines.push_back(move(l));
    }
    for (int i = 0; i < lines.size(); i++) {
        Line &l = lines[i];
        for (l.sz = 1; l.sz < l.verts.size()-1; l.sz <<= 1);
        l.tradd.assign(2*l.sz-1, 0);
        l.trmin.assign(2*l.sz-1, INFCAP);
        for (int j = 0; j < l.verts.size()-1; j++) {
            int v = l.verts[j];
            assert(pheavy[v]);
            l.trmin[l.sz-1 + j] = pcap[v];
        }
        for (int j = l.sz-2; j >= 0; j--)
            l.trmin[j] = min(l.trmin[2*j+1], l.trmin[2*j+2]);
    }

/*    for (int i = 0; i < n; i++) {
        Ef("V%d ->%c%d  in (%d,%d)\n", i, (pheavy[i] ? 'h' : 'l'), par[i], hidx[i], hpos[i]);
    }*/

    int64 totalCost = 0;
    int64 totalFlow = 0;
    double answer = 0.0;
    for (int i = 0; i < k; i++) {
        int minCap = Query(leaves[i].v, 0);
        if (minCap == 0)
            continue;

        int64 addCost = leaves[i].cost * minCap;
        if (totalCost + addCost > 0) {
            assert(addCost > 0);
            answer = totalFlow - double(minCap) * totalCost / addCost;
            break;
        }

        totalCost += addCost;
        totalFlow += minCap;
        answer = double(totalFlow);
        Query(leaves[i].v, -minCap);
    }

    printf("%0.15lf\n", answer);
    return 0;
}
